import importlib.metadata
from typing import Any, Collection
from opentelemetry.instrumentation.instrumentor import BaseInstrumentor
from opentelemetry.trace import get_tracer
from wrapt import wrap_function_wrapper

from obiguard.api_resources.instrumentation.litellm.patch import patch_litellm


class LitellmInstrumentor(BaseInstrumentor):
    methods_to_patch = [
        {
            "module": "litellm",
            "method": "completion",
        },
        {
            "module": "litellm",
            "method": "text_completion",
        },
        {
            "module": "litellm.main",
            "method": "acompletion",
        },
        {
            "module": "litellm.main",
            "method": "image_generation",
        },
        {
            "module": "litellm.main",
            "method": "aimage_generation",
        },
        {
            "module": "litellm.main",
            "method": "embedding",
        },
        {
            "module": "litellm.main",
            "method": "aembedding",
        },
    ]

    def instrumentation_dependencies(self) -> Collection[str]:
        return ["litellm >= 1.48.0"]

    def _instrument(self, **kwargs: Any) -> None:
        version = importlib.metadata.version("litellm")
        tracer_provider = kwargs.get("tracer_provider")
        tracer = get_tracer(__name__, "", tracer_provider)
        for method in self.methods_to_patch:
            wrap_function_wrapper(
                module=method["module"],
                name=method["method"],
                wrapper=patch_litellm(
                    method["module"], method["method"], version, tracer
                ),
            )

    def _uninstrument(self, **kwargs: Any) -> None:
        pass



import json
from typing import Any, Dict, List, Optional
import httpx
from pydantic import BaseModel, PrivateAttr
from obiguard.api_resources.types.utils import parse_headers


class LogsExportCreateResponse(BaseModel, extra="allow"):
    id: Optional[str] = None
    total: Optional[int] = None
    object: Optional[str] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class LogsExportListResponse(BaseModel, extra="allow"):
    object: Optional[str] = None
    total: Optional[int] = None
    data: Optional[List[Dict[str, Any]]] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class LogsExportUpdateResponse(BaseModel, extra="allow"):
    id: Optional[str] = None
    total: Optional[int] = None
    object: Optional[str] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class LogsExportCancelResponse(BaseModel, extra="allow"):
    message: Optional[str] = None
    object: Optional[str] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class LogsExportRetrieveResponse(BaseModel, extra="allow"):
    id: Optional[str] = None
    organisation_id: Optional[str] = None
    filters: Optional[Dict[str, Any]] = None
    requested_data: Optional[List[str]] = None
    status: Optional[str] = None
    description: Optional[str] = None
    created_at: Optional[str] = None
    last_updated_at: Optional[str] = None
    created_by: Optional[str] = None
    workspace_id: Optional[str] = None
    total_records: Optional[int] = None
    object: Optional[str] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class LogsExportStartResponse(BaseModel, extra="allow"):
    message: Optional[str] = None
    object: Optional[str] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class LogsExportDownloadResponse(BaseModel, extra="allow"):
    signed_url: Optional[str] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default



import json
from typing import Any, AsyncIterator, Dict, Iterable, Iterator, List, Optional, Union
from obiguard.api_resources.client import AsyncObiguard, Obiguard
from ..._vendor.openai._types import NotGiven, NOT_GIVEN

# from obiguard.api_resources.utils import TextCompletionChunk
from obiguard.api_resources.types.complete_type import (
    TextCompletion,
    TextCompletionChunk,
)
from obiguard.api_resources.apis.api_resource import APIResource, AsyncAPIResource


class Completion(APIResource):
    def __init__(self, client: Obiguard) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client
        self.client = client

    def stream_create(  # type: ignore[return]
        self,
        model,
        prompt,
        stream,
        temperature,
        max_tokens,
        top_p,
        best_of,
        echo,
        frequency_penalty,
        logit_bias,
        logprobs,
        n,
        presence_penalty,
        seed,
        stop,
        suffix,
        user,
        stream_options,
        **kwargs,
    ) -> Union[TextCompletion, Iterator[TextCompletionChunk]]:
        with self.openai_client.with_streaming_response.completions.create(
            model=model,
            prompt=prompt,
            stream=stream,
            temperature=temperature,
            max_tokens=max_tokens,
            top_p=top_p,
            best_of=best_of,
            echo=echo,
            frequency_penalty=frequency_penalty,
            logit_bias=logit_bias,
            logprobs=logprobs,
            n=n,
            presence_penalty=presence_penalty,
            seed=seed,
            stop=stop,
            suffix=suffix,
            user=user,
            stream_options=stream_options,
            extra_body=kwargs,
        ) as response:
            for line in response.iter_lines():
                json_string = line.replace("data: ", "")
                json_string = json_string.strip().rstrip("\n")
                if json_string == "":
                    continue
                elif json_string == "[DONE]":
                    break
                elif json_string != "":
                    json_data = json.loads(json_string)
                    json_data = TextCompletionChunk(**json_data)
                    yield json_data
                else:
                    return ""

    def normal_create(
        self,
        model,
        prompt,
        stream,
        temperature,
        max_tokens,
        top_p,
        best_of,
        echo,
        frequency_penalty,
        logit_bias,
        logprobs,
        n,
        presence_penalty,
        seed,
        stop,
        suffix,
        user,
        stream_options,
        **kwargs,
    ) -> TextCompletion:
        response = self.openai_client.with_raw_response.completions.create(
            model=model,
            prompt=prompt,
            stream=stream,
            temperature=temperature,
            max_tokens=max_tokens,
            top_p=top_p,
            best_of=best_of,
            echo=echo,
            frequency_penalty=frequency_penalty,
            logit_bias=logit_bias,
            logprobs=logprobs,
            n=n,
            presence_penalty=presence_penalty,
            seed=seed,
            stop=stop,
            suffix=suffix,
            user=user,
            stream_options=stream_options,
            extra_body=kwargs,
        )
        data = TextCompletion(**json.loads(response.text))
        data._headers = response.headers
        return data

    def create(
        self,
        *,
        model: Optional[str] = "portkey-default",
        prompt: Union[
            str, List[str], Iterable[int], Iterable[Iterable[int]], None
        ] = None,
        stream: Union[bool, NotGiven] = NOT_GIVEN,
        temperature: Union[float, NotGiven] = NOT_GIVEN,
        max_tokens: Union[int, NotGiven] = NOT_GIVEN,
        top_p: Union[bool, NotGiven] = NOT_GIVEN,
        best_of: Union[int, NotGiven] = NOT_GIVEN,
        echo: Union[bool, NotGiven] = NOT_GIVEN,
        frequency_penalty: Union[float, NotGiven] = NOT_GIVEN,
        logit_bias: Union[Dict[str, int], NotGiven] = NOT_GIVEN,
        logprobs: Union[int, NotGiven] = NOT_GIVEN,
        n: Union[int, NotGiven] = NOT_GIVEN,
        presence_penalty: Union[float, NotGiven] = NOT_GIVEN,
        seed: Union[int, NotGiven] = NOT_GIVEN,
        stop: Union[Optional[str], List[str], None, NotGiven] = NOT_GIVEN,
        suffix: Union[str, NotGiven] = NOT_GIVEN,
        user: Union[str, NotGiven] = NOT_GIVEN,
        stream_options: Optional[Any] = NOT_GIVEN,
        **kwargs,
    ) -> Union[TextCompletion, Iterator[TextCompletionChunk]]:
        if stream is True:
            return self.stream_create(
                model=model,
                prompt=prompt,
                stream=stream,
                temperature=temperature,
                max_tokens=max_tokens,
                top_p=top_p,
                best_of=best_of,
                echo=echo,
                frequency_penalty=frequency_penalty,
                logit_bias=logit_bias,
                logprobs=logprobs,
                n=n,
                presence_penalty=presence_penalty,
                seed=seed,
                stop=stop,
                suffix=suffix,
                user=user,
                stream_options=stream_options,
                **kwargs,
            )
        else:
            return self.normal_create(
                model=model,
                prompt=prompt,
                stream=stream,
                temperature=temperature,
                max_tokens=max_tokens,
                top_p=top_p,
                best_of=best_of,
                echo=echo,
                frequency_penalty=frequency_penalty,
                logit_bias=logit_bias,
                logprobs=logprobs,
                n=n,
                presence_penalty=presence_penalty,
                seed=seed,
                stop=stop,
                suffix=suffix,
                user=user,
                stream_options=stream_options,
                **kwargs,
            )


class AsyncCompletion(AsyncAPIResource):
    def __init__(self, client: AsyncObiguard) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    async def stream_create(
        self,
        model,
        prompt,
        stream,
        temperature,
        max_tokens,
        top_p,
        best_of,
        echo,
        frequency_penalty,
        logit_bias,
        logprobs,
        n,
        presence_penalty,
        seed,
        stop,
        suffix,
        user,
        stream_options,
        **kwargs,
    ) -> Union[TextCompletion, AsyncIterator[TextCompletionChunk]]:
        async with self.openai_client.with_streaming_response.completions.create(
            model=model,
            prompt=prompt,
            stream=stream,
            temperature=temperature,
            max_tokens=max_tokens,
            top_p=top_p,
            best_of=best_of,
            echo=echo,
            frequency_penalty=frequency_penalty,
            logit_bias=logit_bias,
            logprobs=logprobs,
            n=n,
            presence_penalty=presence_penalty,
            seed=seed,
            stop=stop,
            suffix=suffix,
            user=user,
            stream_options=stream_options,
            extra_body=kwargs,
        ) as response:
            async for line in response.iter_lines():
                json_string = line.replace("data: ", "")
                json_string = json_string.strip().rstrip("\n")
                if json_string == "":
                    continue
                elif json_string == "[DONE]":
                    break
                elif json_string != "":
                    json_data = json.loads(json_string)
                    json_data = TextCompletionChunk(**json_data)
                    yield json_data
                else:
                    pass

    async def normal_create(
        self,
        model,
        prompt,
        stream,
        temperature,
        max_tokens,
        top_p,
        best_of,
        echo,
        frequency_penalty,
        logit_bias,
        logprobs,
        n,
        presence_penalty,
        seed,
        stop,
        suffix,
        user,
        stream_options,
        **kwargs,
    ) -> TextCompletion:
        response = await self.openai_client.with_raw_response.completions.create(
            model=model,
            prompt=prompt,
            stream=stream,
            temperature=temperature,
            max_tokens=max_tokens,
            top_p=top_p,
            best_of=best_of,
            echo=echo,
            frequency_penalty=frequency_penalty,
            logit_bias=logit_bias,
            logprobs=logprobs,
            n=n,
            presence_penalty=presence_penalty,
            seed=seed,
            stop=stop,
            suffix=suffix,
            user=user,
            stream_options=stream_options,
            extra_body=kwargs,
        )
        data = TextCompletion(**json.loads(response.text))
        data._headers = response.headers
        return data

    async def create(
        self,
        *,
        model: Optional[str] = "portkey-default",
        prompt: Union[
            str, List[str], Iterable[int], Iterable[Iterable[int]], None
        ] = None,
        stream: Union[bool, NotGiven] = NOT_GIVEN,
        temperature: Union[float, NotGiven] = NOT_GIVEN,
        max_tokens: Union[int, NotGiven] = NOT_GIVEN,
        top_p: Union[bool, NotGiven] = NOT_GIVEN,
        best_of: Union[int, NotGiven] = NOT_GIVEN,
        echo: Union[bool, NotGiven] = NOT_GIVEN,
        frequency_penalty: Union[float, NotGiven] = NOT_GIVEN,
        logit_bias: Union[Dict[str, int], NotGiven] = NOT_GIVEN,
        logprobs: Union[int, NotGiven] = NOT_GIVEN,
        n: Union[int, NotGiven] = NOT_GIVEN,
        presence_penalty: Union[float, NotGiven] = NOT_GIVEN,
        seed: Union[int, NotGiven] = NOT_GIVEN,
        stop: Union[Optional[str], List[str], None, NotGiven] = NOT_GIVEN,
        suffix: Union[str, NotGiven] = NOT_GIVEN,
        user: Union[str, NotGiven] = NOT_GIVEN,
        stream_options: Optional[Any] = NOT_GIVEN,
        **kwargs,
    ) -> Union[TextCompletion, AsyncIterator[TextCompletionChunk]]:
        if stream is True:
            return self.stream_create(
                model=model,
                prompt=prompt,
                stream=stream,
                temperature=temperature,
                max_tokens=max_tokens,
                top_p=top_p,
                best_of=best_of,
                echo=echo,
                frequency_penalty=frequency_penalty,
                logit_bias=logit_bias,
                logprobs=logprobs,
                n=n,
                presence_penalty=presence_penalty,
                seed=seed,
                stop=stop,
                suffix=suffix,
                user=user,
                stream_options=stream_options,
                **kwargs,
            )
        else:
            return await self.normal_create(
                model=model,
                prompt=prompt,
                stream=stream,
                temperature=temperature,
                max_tokens=max_tokens,
                top_p=top_p,
                best_of=best_of,
                echo=echo,
                frequency_penalty=frequency_penalty,
                logit_bias=logit_bias,
                logprobs=logprobs,
                n=n,
                presence_penalty=presence_penalty,
                seed=seed,
                stop=stop,
                suffix=suffix,
                user=user,
                stream_options=stream_options,
                **kwargs,
            )



from typing import Any, Optional
from urllib.parse import urlencode
from obiguard.api_resources.apis.api_resource import APIResource, AsyncAPIResource
from obiguard.api_resources.base_client import APIClient, AsyncAPIClient
from obiguard.api_resources.utils import GenericResponse, PortkeyApiPaths


class Labels(APIResource):
    def __init__(self, client: APIClient) -> None:
        super().__init__(client)

    def create(
        self,
        *,
        name: str,
        organization_id: Optional[str] = None,
        workspace_id: Optional[str] = None,
        description: Optional[str] = None,
        color_code: Optional[str] = None,
    ) -> Any:
        body = {
            "name": name,
            "organization_id": organization_id,
            "workspace_id": workspace_id,
            "description": description,
            "color_code": color_code,
        }
        return self._post(
            f"{PortkeyApiPaths.LABELS_API}",
            params=None,
            body=body,
            cast_to=GenericResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )

    def list(
        self,
        *,
        organization_id: Optional[str] = None,
        workspace_id: Optional[str] = None,
        current_page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> Any:
        query = {
            "organization_id": organization_id,
            "workspace_id": workspace_id,
            "current_page": current_page,
            "page_size": page_size,
        }
        filtered_query = {k: v for k, v in query.items() if v is not None}
        query_string = urlencode(filtered_query)
        return self._get(
            f"{PortkeyApiPaths.LABELS_API}?{query_string}",
            params=None,
            body=None,
            cast_to=GenericResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )

    def retrieve(
        self,
        label_id: str,
    ) -> Any:
        return self._get(
            f"{PortkeyApiPaths.LABELS_API}/{label_id}",
            params=None,
            body=None,
            cast_to=GenericResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )

    def update(
        self,
        label_id: str,
        *,
        name: Optional[str] = None,
        description: Optional[str] = None,
        color_code: Optional[str] = None,
    ) -> Any:
        body = {
            "name": name,
            "description": description,
            "color_code": color_code,
        }
        return self._put(
            f"{PortkeyApiPaths.LABELS_API}/{label_id}",
            params=None,
            body=body,
            cast_to=GenericResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )

    def delete(
        self,
        label_id: str,
    ) -> Any:
        return self._delete(
            f"{PortkeyApiPaths.LABELS_API}/{label_id}",
            params=None,
            body=None,
            cast_to=GenericResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )


class AsyncLabels(AsyncAPIResource):
    def __init__(self, client: AsyncAPIClient) -> None:
        super().__init__(client)

    async def create(
        self,
        *,
        name: str,
        organization_id: Optional[str] = None,
        workspace_id: Optional[str] = None,
        description: Optional[str] = None,
        color_code: Optional[str] = None,
    ) -> Any:
        body = {
            "name": name,
            "organization_id": organization_id,
            "workspace_id": workspace_id,
            "description": description,
            "color_code": color_code,
        }
        return await self._post(
            f"{PortkeyApiPaths.LABELS_API}",
            params=None,
            body=body,
            cast_to=GenericResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )

    async def list(
        self,
        *,
        organization_id: Optional[str] = None,
        workspace_id: Optional[str] = None,
        current_page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> Any:
        query = {
            "organization_id": organization_id,
            "workspace_id": workspace_id,
            "current_page": current_page,
            "page_size": page_size,
        }
        filtered_query = {k: v for k, v in query.items() if v is not None}
        query_string = urlencode(filtered_query)
        return await self._get(
            f"{PortkeyApiPaths.LABELS_API}?{query_string}",
            params=None,
            body=None,
            cast_to=GenericResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )

    async def retrieve(
        self,
        label_id: str,
    ) -> Any:
        return await self._get(
            f"{PortkeyApiPaths.LABELS_API}/{label_id}",
            params=None,
            body=None,
            cast_to=GenericResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )

    async def update(
        self,
        label_id: str,
        *,
        name: Optional[str] = None,
        description: Optional[str] = None,
        color_code: Optional[str] = None,
    ) -> Any:
        body = {
            "name": name,
            "description": description,
            "color_code": color_code,
        }
        return await self._put(
            f"{PortkeyApiPaths.LABELS_API}/{label_id}",
            params=None,
            body=body,
            cast_to=GenericResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )

    async def delete(
        self,
        label_id: str,
    ) -> Any:
        return await self._delete(
            f"{PortkeyApiPaths.LABELS_API}/{label_id}",
            params=None,
            body=None,
            cast_to=GenericResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )



import json
from typing import Dict, List, Optional, Union
from typing_extensions import Annotated, TypeAlias
import httpx

from obiguard._vendor.openai._utils._transform import PropertyInfo
from .utils import parse_headers
from pydantic import BaseModel, PrivateAttr

__all__ = [
    "LastError",
    "ExpiresAfter",
    "VectorStore",
    "VectorStoreList",
    "VectorStoreDeleted",
    "VectorStoreFile",
    "VectorStoreFileList",
    "VectorStoreFileDeleted",
    "FileCounts",
    "VectorStoreFileBatch",
]


class FileCounts(BaseModel, extra="allow"):
    cancelled: Optional[int] = None
    completed: Optional[int] = None
    failed: Optional[int] = None
    in_progress: Optional[int] = None
    total: Optional[int] = None


class ExpiresAfter(BaseModel, extra="allow"):
    anchor: Optional[str] = None
    days: Optional[int] = None


class VectorStore(BaseModel, extra="allow"):
    id: Optional[str] = None
    created_at: Optional[int] = None
    file_counts: Optional[FileCounts] = None
    last_active_at: Optional[int] = None
    metadata: Optional[object] = None
    name: Optional[str] = None
    object: Optional[str] = None
    status: Optional[str] = None
    usage_bytes: Optional[int] = None
    expires_after: Optional[ExpiresAfter] = None
    expires_at: Optional[int] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class VectorStoreList(BaseModel, extra="allow"):
    data: Optional[List[VectorStore]] = None
    object: Optional[str] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class VectorStoreDeleted(BaseModel, extra="allow"):
    id: Optional[str] = None
    deleted: Optional[bool] = None
    object: Optional[str] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class LastError(BaseModel, extra="allow"):
    code: Optional[str] = None
    message: Optional[str] = None


class StaticFileChunkingStrategy(BaseModel, extra="allow"):
    chunk_overlap_tokens: Optional[int] = None
    max_chunk_size_tokens: Optional[int] = None


class StaticFileChunkingStrategyObject(BaseModel, extra="allow"):
    static: StaticFileChunkingStrategy
    type: Optional[str] = None


class OtherFileChunkingStrategyObject(BaseModel, extra="allow"):
    type: Optional[str] = None


ChunkingStrategy: TypeAlias = Annotated[
    Union[StaticFileChunkingStrategyObject, OtherFileChunkingStrategyObject],
    PropertyInfo(discriminator="type"),
]


class VectorStoreFile(BaseModel, extra="allow"):
    id: Optional[str] = None
    created_at: Optional[int] = None
    last_error: Optional[LastError] = None
    object: Optional[str] = None
    status: Optional[str] = None
    usage_bytes: Optional[int] = None
    vector_store_id: Optional[str] = None
    chunking_strategy: Optional[ChunkingStrategy] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class VectorStoreFileList(BaseModel, extra="allow"):
    data: Optional[List[VectorStoreFile]] = None
    object: Optional[str] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class VectorStoreFileDeleted(BaseModel, extra="allow"):
    id: Optional[str] = None
    deleted: Optional[bool] = None
    object: Optional[str] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class VectorStoreFileBatch(BaseModel, extra="allow"):
    id: Optional[str] = None
    created_at: Optional[int] = None
    file_counts: Optional[FileCounts] = None
    object: Optional[str] = None
    status: Optional[str] = None
    vector_store_id: Optional[str] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)



import json
from typing import Iterable, List, Literal, Optional, Union
from obiguard._vendor.openai._streaming import AsyncStream, Stream
from obiguard._vendor.openai.lib._parsing._responses import TextFormatT
from obiguard._vendor.openai.lib.streaming.responses._responses import (
    AsyncResponseStreamManager,
    ResponseStreamManager,
)
from obiguard._vendor.openai.types.responses import response_create_params
from obiguard._vendor.openai.types.responses.parsed_response import ParsedResponse
from obiguard._vendor.openai.types.responses.response_includable import (
    ResponseIncludable,
)
from obiguard._vendor.openai.types.responses.response_input_param import (
    ResponseInputParam,
)
from obiguard._vendor.openai.types.responses.response_stream_event import (
    ResponseStreamEvent,
)
from obiguard._vendor.openai.types.responses.response_text_config_param import (
    ResponseTextConfigParam,
)
from obiguard._vendor.openai.types.responses.tool_param import (
    ParseableToolParam,
    ToolParam,
)
from obiguard._vendor.openai.types.responses.response import Response
from obiguard._vendor.openai.types.shared.chat_model import ChatModel
from obiguard._vendor.openai.types.shared.responses_model import ResponsesModel
from obiguard._vendor.openai.types.shared_params.reasoning import Reasoning
from obiguard.api_resources.apis.api_resource import APIResource, AsyncAPIResource
from obiguard.api_resources.client import AsyncObiguard, Obiguard
from obiguard.api_resources.types.response_type import Response as ResponseType
from obiguard.api_resources.types.responses_input_items_type import InputItemList
from obiguard.api_resources.types.shared_types import Metadata
from ..._vendor.openai._types import NotGiven, NOT_GIVEN
from typing_extensions import overload


class Responses(APIResource):
    def __init__(self, client: Obiguard) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client
        self.input_items = InputItems(client)

    @overload
    def create(
        self,
        *,
        input: Union[str, ResponseInputParam],
        model: ResponsesModel,
        include: Union[Optional[List[ResponseIncludable]], NotGiven] = NOT_GIVEN,
        instructions: Union[Optional[str], NotGiven] = NOT_GIVEN,
        max_output_tokens: Union[Optional[int], NotGiven] = NOT_GIVEN,
        metadata: Union[Optional[Metadata], NotGiven] = NOT_GIVEN,
        parallel_tool_calls: Union[Optional[bool], NotGiven] = NOT_GIVEN,
        previous_response_id: Union[Optional[str], NotGiven] = NOT_GIVEN,
        reasoning: Union[Optional[Reasoning], NotGiven] = NOT_GIVEN,
        store: Union[Optional[bool], NotGiven] = NOT_GIVEN,
        stream: Union[Literal[False], NotGiven] = NOT_GIVEN,
        temperature: Union[Optional[float], NotGiven] = NOT_GIVEN,
        text: Union[ResponseTextConfigParam, NotGiven] = NOT_GIVEN,
        tool_choice: Union[response_create_params.ToolChoice, NotGiven] = NOT_GIVEN,
        tools: Union[Iterable[ToolParam], NotGiven] = NOT_GIVEN,
        top_p: Union[Optional[float], NotGiven] = NOT_GIVEN,
        truncation: Union[Optional[Literal["auto", "disabled"]], NotGiven] = NOT_GIVEN,
        user: Union[str, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> Response:
        ...

    @overload
    def create(
        self,
        *,
        input: Union[str, ResponseInputParam],
        model: ResponsesModel,
        stream: Literal[True],
        include: Union[Optional[List[ResponseIncludable]], NotGiven] = NOT_GIVEN,
        instructions: Union[Optional[str], NotGiven] = NOT_GIVEN,
        max_output_tokens: Union[Optional[int], NotGiven] = NOT_GIVEN,
        metadata: Union[Optional[Metadata], NotGiven] = NOT_GIVEN,
        parallel_tool_calls: Union[Optional[bool], NotGiven] = NOT_GIVEN,
        previous_response_id: Union[Optional[str], NotGiven] = NOT_GIVEN,
        reasoning: Union[Optional[Reasoning], NotGiven] = NOT_GIVEN,
        store: Union[Optional[bool], NotGiven] = NOT_GIVEN,
        temperature: Union[Optional[float], NotGiven] = NOT_GIVEN,
        text: Union[ResponseTextConfigParam, NotGiven] = NOT_GIVEN,
        tool_choice: Union[response_create_params.ToolChoice, NotGiven] = NOT_GIVEN,
        tools: Union[Iterable[ToolParam], NotGiven] = NOT_GIVEN,
        top_p: Union[Optional[float], NotGiven] = NOT_GIVEN,
        truncation: Union[Optional[Literal["auto", "disabled"]], NotGiven] = NOT_GIVEN,
        user: Union[str, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> Stream[ResponseStreamEvent]:
        ...

    @overload
    def create(
        self,
        *,
        input: Union[str, ResponseInputParam],
        model: ResponsesModel,
        stream: bool,
        include: Union[Optional[List[ResponseIncludable]], NotGiven] = NOT_GIVEN,
        instructions: Union[Optional[str], NotGiven] = NOT_GIVEN,
        max_output_tokens: Union[Optional[int], NotGiven] = NOT_GIVEN,
        metadata: Union[Optional[Metadata], NotGiven] = NOT_GIVEN,
        parallel_tool_calls: Union[Optional[bool], NotGiven] = NOT_GIVEN,
        previous_response_id: Union[Optional[str], NotGiven] = NOT_GIVEN,
        reasoning: Union[Optional[Reasoning], NotGiven] = NOT_GIVEN,
        store: Union[Optional[bool], NotGiven] = NOT_GIVEN,
        temperature: Union[Optional[float], NotGiven] = NOT_GIVEN,
        text: Union[ResponseTextConfigParam, NotGiven] = NOT_GIVEN,
        tool_choice: Union[response_create_params.ToolChoice, NotGiven] = NOT_GIVEN,
        tools: Union[Iterable[ToolParam], NotGiven] = NOT_GIVEN,
        top_p: Union[Optional[float], NotGiven] = NOT_GIVEN,
        truncation: Union[Optional[Literal["auto", "disabled"]], NotGiven] = NOT_GIVEN,
        user: Union[str, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> Union[Response, Stream[ResponseStreamEvent]]:
        ...

    def create(
        self,
        *,
        input: Union[str, ResponseInputParam],
        model: ResponsesModel,
        include: Union[Optional[List[ResponseIncludable]], NotGiven] = NOT_GIVEN,
        instructions: Union[Optional[str], NotGiven] = NOT_GIVEN,
        max_output_tokens: Union[Optional[int], NotGiven] = NOT_GIVEN,
        metadata: Union[Optional[Metadata], NotGiven] = NOT_GIVEN,
        parallel_tool_calls: Union[Optional[bool], NotGiven] = NOT_GIVEN,
        previous_response_id: Union[Optional[str], NotGiven] = NOT_GIVEN,
        reasoning: Union[Optional[Reasoning], NotGiven] = NOT_GIVEN,
        store: Union[Optional[bool], NotGiven] = NOT_GIVEN,
        stream: Union[Optional[Literal[False]], Literal[True], NotGiven] = NOT_GIVEN,
        temperature: Union[Optional[float], NotGiven] = NOT_GIVEN,
        text: Union[ResponseTextConfigParam, NotGiven] = NOT_GIVEN,
        tool_choice: Union[response_create_params.ToolChoice, NotGiven] = NOT_GIVEN,
        tools: Union[Iterable[ToolParam], NotGiven] = NOT_GIVEN,
        top_p: Union[Optional[float], NotGiven] = NOT_GIVEN,
        truncation: Union[Optional[Literal["auto", "disabled"]], NotGiven] = NOT_GIVEN,
        user: Union[str, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> Union[Response, Stream[ResponseStreamEvent]]:
        extra_headers = kwargs.pop("extra_headers", None)
        extra_query = kwargs.pop("extra_query", None)
        extra_body = kwargs.pop("extra_body", None)
        timeout = kwargs.pop("timeout", None)

        return self.openai_client.responses.create(  # type: ignore[misc]
            input=input,
            model=model,
            include=include,
            instructions=instructions,
            max_output_tokens=max_output_tokens,
            metadata=metadata,
            parallel_tool_calls=parallel_tool_calls,
            previous_response_id=previous_response_id,
            reasoning=reasoning,
            store=store,
            stream=stream,  # type: ignore[arg-type]
            temperature=temperature,
            text=text,
            tool_choice=tool_choice,
            tools=tools,
            top_p=top_p,
            truncation=truncation,
            user=user,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

    def retrieve(
        self,
        response_id: str,
        *,
        include: Union[List[ResponseIncludable], NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> ResponseType:
        extra_headers = kwargs.pop("extra_headers", None)
        extra_query = kwargs.pop("extra_query", None)
        extra_body = kwargs.pop("extra_body", None)
        timeout = kwargs.pop("timeout", None)

        response = self.openai_client.with_raw_response.responses.retrieve(
            response_id=response_id,
            include=include,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

        data = ResponseType(**json.loads(response.text))
        data._headers = response.headers

        return data

    def delete(self, response_id: str, **kwargs) -> None:
        extra_headers = kwargs.pop("extra_headers", None)
        extra_query = kwargs.pop("extra_query", None)
        extra_body = kwargs.pop("extra_body", None)
        timeout = kwargs.pop("timeout", None)

        return self.openai_client.responses.delete(
            response_id=response_id,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

    def stream(
        self,
        *,
        input: Union[str, ResponseInputParam],
        model: Union[str, ChatModel],
        text_format: Union[type[TextFormatT], NotGiven] = NOT_GIVEN,  # type: ignore[type-arg]
        tools: Union[Iterable[ParseableToolParam], NotGiven] = NOT_GIVEN,
        include: Union[List[ResponseIncludable], NotGiven] = NOT_GIVEN,
        instructions: Union[str, NotGiven] = NOT_GIVEN,
        max_output_tokens: Union[int, NotGiven] = NOT_GIVEN,
        metadata: Union[Metadata, NotGiven] = NOT_GIVEN,
        parallel_tool_calls: Union[bool, NotGiven] = NOT_GIVEN,
        previous_response_id: Union[str, NotGiven] = NOT_GIVEN,
        reasoning: Union[Reasoning, NotGiven] = NOT_GIVEN,
        store: Union[bool, NotGiven] = NOT_GIVEN,
        temperature: Union[float, NotGiven] = NOT_GIVEN,
        text: Union[ResponseTextConfigParam, NotGiven] = NOT_GIVEN,
        tool_choice: Union[response_create_params.ToolChoice, NotGiven] = NOT_GIVEN,
        top_p: Union[float, NotGiven] = NOT_GIVEN,
        truncation: Union[Literal["auto", "disabled"], NotGiven] = NOT_GIVEN,
        user: Union[str, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> ResponseStreamManager[TextFormatT]:
        extra_headers = kwargs.pop("extra_headers", None)
        extra_query = kwargs.pop("extra_query", None)
        extra_body = kwargs.pop("extra_body", None)
        timeout = kwargs.pop("timeout", None)
        return self.openai_client.responses.stream(
            input=input,
            model=model,
            text_format=text_format,
            tools=tools,
            include=include,
            instructions=instructions,
            max_output_tokens=max_output_tokens,
            metadata=metadata,
            parallel_tool_calls=parallel_tool_calls,
            previous_response_id=previous_response_id,
            reasoning=reasoning,
            store=store,
            temperature=temperature,
            text=text,
            tool_choice=tool_choice,
            top_p=top_p,
            truncation=truncation,
            user=user,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

    def parse(
        self,
        *,
        input: Union[str, ResponseInputParam],
        model: Union[str, ChatModel],
        text_format: Union[type[TextFormatT], NotGiven] = NOT_GIVEN,  # type: ignore[type-arg]
        tools: Union[Iterable[ParseableToolParam], NotGiven] = NOT_GIVEN,
        include: Union[List[ResponseIncludable], NotGiven] = NOT_GIVEN,
        instructions: Union[str, NotGiven] = NOT_GIVEN,
        max_output_tokens: Union[int, NotGiven] = NOT_GIVEN,
        metadata: Union[Metadata, NotGiven] = NOT_GIVEN,
        parallel_tool_calls: Union[bool, NotGiven] = NOT_GIVEN,
        previous_response_id: Union[str, NotGiven] = NOT_GIVEN,
        reasoning: Union[Reasoning, NotGiven] = NOT_GIVEN,
        store: Union[bool, NotGiven] = NOT_GIVEN,
        stream: Union[Literal[False], Literal[True], NotGiven] = NOT_GIVEN,
        temperature: Union[float, NotGiven] = NOT_GIVEN,
        text: Union[ResponseTextConfigParam, NotGiven] = NOT_GIVEN,
        tool_choice: Union[response_create_params.ToolChoice, NotGiven] = NOT_GIVEN,
        top_p: Union[float, NotGiven] = NOT_GIVEN,
        truncation: Union[Literal["auto", "disabled"], NotGiven] = NOT_GIVEN,
        user: Union[str, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> ParsedResponse[TextFormatT]:
        extra_headers = kwargs.pop("extra_headers", None)
        extra_query = kwargs.pop("extra_query", None)
        extra_body = kwargs.pop("extra_body", None)
        timeout = kwargs.pop("timeout", None)

        return self.openai_client.responses.parse(
            input=input,
            model=model,
            text_format=text_format,
            tools=tools,
            include=include,
            instructions=instructions,
            max_output_tokens=max_output_tokens,
            metadata=metadata,
            parallel_tool_calls=parallel_tool_calls,
            previous_response_id=previous_response_id,
            reasoning=reasoning,
            store=store,
            stream=stream,
            temperature=temperature,
            text=text,
            tool_choice=tool_choice,
            top_p=top_p,
            truncation=truncation,
            user=user,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )


class InputItems(APIResource):
    def __init__(self, client: Obiguard) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    def list(
        self,
        response_id: str,
        *,
        after: Union[str, NotGiven] = NOT_GIVEN,
        before: Union[str, NotGiven] = NOT_GIVEN,
        include: Union[List[ResponseIncludable], NotGiven] = NOT_GIVEN,
        limit: Union[int, NotGiven] = NOT_GIVEN,
        order: Union[Literal["asc", "desc"], NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> InputItemList:
        extra_headers = kwargs.pop("extra_headers", None)
        extra_query = kwargs.pop("extra_query", None)
        extra_body = kwargs.pop("extra_body", None)
        timeout = kwargs.pop("timeout", None)
        response = self.openai_client.responses.input_items.list(
            response_id=response_id,
            after=after,
            before=before,
            include=include,
            limit=limit,
            order=order,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

        return response  # type: ignore[return-value]


class AsyncResponses(AsyncAPIResource):
    def __init__(self, client: AsyncObiguard) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client
        self.input_items = AsyncInputItems(client)

    @overload
    async def create(
        self,
        *,
        input: Union[str, ResponseInputParam],
        model: ResponsesModel,
        include: Union[Optional[List[ResponseIncludable]], NotGiven] = NOT_GIVEN,
        instructions: Union[Optional[str], NotGiven] = NOT_GIVEN,
        max_output_tokens: Union[Optional[int], NotGiven] = NOT_GIVEN,
        metadata: Union[Optional[Metadata], NotGiven] = NOT_GIVEN,
        parallel_tool_calls: Union[Optional[bool], NotGiven] = NOT_GIVEN,
        previous_response_id: Union[Optional[str], NotGiven] = NOT_GIVEN,
        reasoning: Union[Optional[Reasoning], NotGiven] = NOT_GIVEN,
        store: Union[Optional[bool], NotGiven] = NOT_GIVEN,
        stream: Union[Literal[False], NotGiven] = NOT_GIVEN,
        temperature: Union[Optional[float], NotGiven] = NOT_GIVEN,
        text: Union[ResponseTextConfigParam, NotGiven] = NOT_GIVEN,
        tool_choice: Union[response_create_params.ToolChoice, NotGiven] = NOT_GIVEN,
        tools: Union[Iterable[ToolParam], NotGiven] = NOT_GIVEN,
        top_p: Union[Optional[float], NotGiven] = NOT_GIVEN,
        truncation: Union[Optional[Literal["auto", "disabled"]], NotGiven] = NOT_GIVEN,
        user: Union[str, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> Response:
        ...

    @overload
    async def create(
        self,
        *,
        input: Union[str, ResponseInputParam],
        model: ResponsesModel,
        stream: Literal[True],
        include: Union[Optional[List[ResponseIncludable]], NotGiven] = NOT_GIVEN,
        instructions: Union[Optional[str], NotGiven] = NOT_GIVEN,
        max_output_tokens: Union[Optional[int], NotGiven] = NOT_GIVEN,
        metadata: Union[Optional[Metadata], NotGiven] = NOT_GIVEN,
        parallel_tool_calls: Union[Optional[bool], NotGiven] = NOT_GIVEN,
        previous_response_id: Union[Optional[str], NotGiven] = NOT_GIVEN,
        reasoning: Union[Optional[Reasoning], NotGiven] = NOT_GIVEN,
        store: Union[Optional[bool], NotGiven] = NOT_GIVEN,
        temperature: Union[Optional[float], NotGiven] = NOT_GIVEN,
        text: Union[ResponseTextConfigParam, NotGiven] = NOT_GIVEN,
        tool_choice: Union[response_create_params.ToolChoice, NotGiven] = NOT_GIVEN,
        tools: Union[Iterable[ToolParam], NotGiven] = NOT_GIVEN,
        top_p: Union[Optional[float], NotGiven] = NOT_GIVEN,
        truncation: Union[Optional[Literal["auto", "disabled"]], NotGiven] = NOT_GIVEN,
        user: Union[str, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> AsyncStream[ResponseStreamEvent]:
        ...

    @overload
    async def create(
        self,
        *,
        input: Union[str, ResponseInputParam],
        model: ResponsesModel,
        stream: bool,
        include: Union[Optional[List[ResponseIncludable]], NotGiven] = NOT_GIVEN,
        instructions: Union[Optional[str], NotGiven] = NOT_GIVEN,
        max_output_tokens: Union[Optional[int], NotGiven] = NOT_GIVEN,
        metadata: Union[Optional[Metadata], NotGiven] = NOT_GIVEN,
        parallel_tool_calls: Union[Optional[bool], NotGiven] = NOT_GIVEN,
        previous_response_id: Union[Optional[str], NotGiven] = NOT_GIVEN,
        reasoning: Union[Optional[Reasoning], NotGiven] = NOT_GIVEN,
        store: Union[Optional[bool], NotGiven] = NOT_GIVEN,
        temperature: Union[Optional[float], NotGiven] = NOT_GIVEN,
        text: Union[ResponseTextConfigParam, NotGiven] = NOT_GIVEN,
        tool_choice: Union[response_create_params.ToolChoice, NotGiven] = NOT_GIVEN,
        tools: Union[Iterable[ToolParam], NotGiven] = NOT_GIVEN,
        top_p: Union[Optional[float], NotGiven] = NOT_GIVEN,
        truncation: Union[Optional[Literal["auto", "disabled"]], NotGiven] = NOT_GIVEN,
        user: Union[str, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> Union[Response, AsyncStream[ResponseStreamEvent]]:
        ...

    async def create(
        self,
        *,
        input: Union[str, ResponseInputParam],
        model: ResponsesModel,
        include: Union[Optional[List[ResponseIncludable]], NotGiven] = NOT_GIVEN,
        instructions: Union[Optional[str], NotGiven] = NOT_GIVEN,
        max_output_tokens: Union[Optional[int], NotGiven] = NOT_GIVEN,
        metadata: Union[Optional[Metadata], NotGiven] = NOT_GIVEN,
        parallel_tool_calls: Union[Optional[bool], NotGiven] = NOT_GIVEN,
        previous_response_id: Union[Optional[str], NotGiven] = NOT_GIVEN,
        reasoning: Union[Optional[Reasoning], NotGiven] = NOT_GIVEN,
        store: Union[Optional[bool], NotGiven] = NOT_GIVEN,
        stream: Union[Optional[Literal[False]], Literal[True], NotGiven] = NOT_GIVEN,
        temperature: Union[Optional[float], NotGiven] = NOT_GIVEN,
        text: Union[ResponseTextConfigParam, NotGiven] = NOT_GIVEN,
        tool_choice: Union[response_create_params.ToolChoice, NotGiven] = NOT_GIVEN,
        tools: Union[Iterable[ToolParam], NotGiven] = NOT_GIVEN,
        top_p: Union[Optional[float], NotGiven] = NOT_GIVEN,
        truncation: Union[Optional[Literal["auto", "disabled"]], NotGiven] = NOT_GIVEN,
        user: Union[str, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> Union[Response, AsyncStream[ResponseStreamEvent]]:
        extra_headers = kwargs.pop("extra_headers", None)
        extra_query = kwargs.pop("extra_query", None)
        extra_body = kwargs.pop("extra_body", None)
        timeout = kwargs.pop("timeout", None)

        return await self.openai_client.responses.create(  # type: ignore[misc]
            input=input,
            model=model,
            include=include,
            instructions=instructions,
            max_output_tokens=max_output_tokens,
            metadata=metadata,
            parallel_tool_calls=parallel_tool_calls,
            previous_response_id=previous_response_id,
            reasoning=reasoning,
            store=store,
            stream=stream,  # type: ignore[arg-type]
            temperature=temperature,
            text=text,
            tool_choice=tool_choice,
            tools=tools,
            top_p=top_p,
            truncation=truncation,
            user=user,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

    async def retrieve(
        self,
        response_id: str,
        *,
        include: Union[List[ResponseIncludable], NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> ResponseType:
        extra_headers = kwargs.pop("extra_headers", None)
        extra_query = kwargs.pop("extra_query", None)
        extra_body = kwargs.pop("extra_body", None)
        timeout = kwargs.pop("timeout", None)

        response = await self.openai_client.with_raw_response.responses.retrieve(
            response_id=response_id,
            include=include,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

        data = ResponseType(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def delete(self, response_id: str, **kwargs) -> None:
        extra_headers = kwargs.pop("extra_headers", None)
        extra_query = kwargs.pop("extra_query", None)
        extra_body = kwargs.pop("extra_body", None)
        timeout = kwargs.pop("timeout", None)

        return await self.openai_client.responses.delete(
            response_id=response_id,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

    def stream(
        self,
        *,
        input: Union[str, ResponseInputParam],
        model: Union[str, ChatModel],
        text_format: Union[type[TextFormatT], NotGiven] = NOT_GIVEN,  # type: ignore[type-arg]
        tools: Union[Iterable[ParseableToolParam], NotGiven] = NOT_GIVEN,
        include: Union[List[ResponseIncludable], NotGiven] = NOT_GIVEN,
        instructions: Union[str, NotGiven] = NOT_GIVEN,
        max_output_tokens: Union[int, NotGiven] = NOT_GIVEN,
        metadata: Union[Metadata, NotGiven] = NOT_GIVEN,
        parallel_tool_calls: Union[bool, NotGiven] = NOT_GIVEN,
        previous_response_id: Union[str, NotGiven] = NOT_GIVEN,
        reasoning: Union[Reasoning, NotGiven] = NOT_GIVEN,
        store: Union[bool, NotGiven] = NOT_GIVEN,
        temperature: Union[float, NotGiven] = NOT_GIVEN,
        text: Union[ResponseTextConfigParam, NotGiven] = NOT_GIVEN,
        tool_choice: Union[response_create_params.ToolChoice, NotGiven] = NOT_GIVEN,
        top_p: Union[float, NotGiven] = NOT_GIVEN,
        truncation: Union[Literal["auto", "disabled"], NotGiven] = NOT_GIVEN,
        user: Union[str, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> AsyncResponseStreamManager[TextFormatT]:
        extra_headers = kwargs.pop("extra_headers", None)
        extra_query = kwargs.pop("extra_query", None)
        extra_body = kwargs.pop("extra_body", None)
        timeout = kwargs.pop("timeout", None)
        return self.openai_client.responses.stream(
            input=input,
            model=model,
            text_format=text_format,
            tools=tools,
            include=include,
            instructions=instructions,
            max_output_tokens=max_output_tokens,
            metadata=metadata,
            parallel_tool_calls=parallel_tool_calls,
            previous_response_id=previous_response_id,
            reasoning=reasoning,
            store=store,
            temperature=temperature,
            text=text,
            tool_choice=tool_choice,
            top_p=top_p,
            truncation=truncation,
            user=user,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

    async def parse(
        self,
        *,
        input: Union[str, ResponseInputParam],
        model: Union[str, ChatModel],
        text_format: Union[type[TextFormatT], NotGiven] = NOT_GIVEN,  # type: ignore[type-arg]
        tools: Union[Iterable[ParseableToolParam], NotGiven] = NOT_GIVEN,
        include: Union[List[ResponseIncludable], NotGiven] = NOT_GIVEN,
        instructions: Union[str, NotGiven] = NOT_GIVEN,
        max_output_tokens: Union[int, NotGiven] = NOT_GIVEN,
        metadata: Union[Metadata, NotGiven] = NOT_GIVEN,
        parallel_tool_calls: Union[bool, NotGiven] = NOT_GIVEN,
        previous_response_id: Union[str, NotGiven] = NOT_GIVEN,
        reasoning: Union[Reasoning, NotGiven] = NOT_GIVEN,
        store: Union[bool, NotGiven] = NOT_GIVEN,
        stream: Union[Literal[False], Literal[True], NotGiven] = NOT_GIVEN,
        temperature: Union[float, NotGiven] = NOT_GIVEN,
        text: Union[ResponseTextConfigParam, NotGiven] = NOT_GIVEN,
        tool_choice: Union[response_create_params.ToolChoice, NotGiven] = NOT_GIVEN,
        top_p: Union[float, NotGiven] = NOT_GIVEN,
        truncation: Union[Literal["auto", "disabled"], NotGiven] = NOT_GIVEN,
        user: Union[str, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> ParsedResponse[TextFormatT]:
        extra_headers = kwargs.pop("extra_headers", None)
        extra_query = kwargs.pop("extra_query", None)
        extra_body = kwargs.pop("extra_body", None)
        timeout = kwargs.pop("timeout", None)

        return await self.openai_client.responses.parse(
            input=input,
            model=model,
            text_format=text_format,
            tools=tools,
            include=include,
            instructions=instructions,
            max_output_tokens=max_output_tokens,
            metadata=metadata,
            parallel_tool_calls=parallel_tool_calls,
            previous_response_id=previous_response_id,
            reasoning=reasoning,
            store=store,
            stream=stream,
            temperature=temperature,
            text=text,
            tool_choice=tool_choice,
            top_p=top_p,
            truncation=truncation,
            user=user,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )


class AsyncInputItems(AsyncAPIResource):
    def __init__(self, client: AsyncObiguard) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    async def list(
        self,
        response_id: str,
        *,
        after: Union[str, NotGiven] = NOT_GIVEN,
        before: Union[str, NotGiven] = NOT_GIVEN,
        include: Union[List[ResponseIncludable], NotGiven] = NOT_GIVEN,
        limit: Union[int, NotGiven] = NOT_GIVEN,
        order: Union[Literal["asc", "desc"], NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> InputItemList:
        extra_headers = kwargs.pop("extra_headers", None)
        extra_query = kwargs.pop("extra_query", None)
        extra_body = kwargs.pop("extra_body", None)
        timeout = kwargs.pop("timeout", None)
        response = await self.openai_client.responses.input_items.list(
            response_id=response_id,
            after=after,
            before=before,
            include=include,
            limit=limit,
            order=order,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

        return response  # type: ignore[return-value]



from opentelemetry import trace
from opentelemetry.trace import SpanKind, Status, StatusCode

from obiguard.utils.json_utils import serialize_args, serialize_kwargs
from obiguard.api_resources.instrumentation.utils import (
    set_span_attribute,
    set_members,
)
from obiguard.utils import string_to_uuid


def patch_litellm(module: str, operation_name: str, version: str, tracer: trace.Tracer):
    def traced_func(wrapped, instance, args, kwargs):
        with tracer.start_as_current_span(
            name=operation_name, kind=SpanKind.CLIENT
        ) as span:
            try:
                module_name = module
                class_name = operation_name

                if operation_name == "completion":
                    headers = kwargs.get("headers", {})
                    headers["x-obiguard-trace-id"] = str(
                        string_to_uuid(span.get_span_context().trace_id)
                    )
                    headers["x-obiguard-span-id"] = str(span.get_span_context().span_id)
                    headers["x-obiguard-parent-span-id"] = (
                        str(span.parent.span_id) if span.parent else None
                    )
                    headers["x-obiguard-span-name"] = span.name
                    kwargs["headers"] = headers

                span.set_attribute("framework.version", version)
                span.set_attribute("_source", "litellm")
                span.set_attribute("_source_type", "routing library")
                span.set_attribute("module", module_name)
                span.set_attribute("method", operation_name)
                span.set_attribute("args", serialize_args(*args))
                span.set_attribute("kwargs", serialize_kwargs(**kwargs))

                result = wrapped(*args, **kwargs)
                span.set_status(Status(StatusCode.OK))

                try:
                    set_members(span, instance, module_name, class_name)
                except Exception as e:
                    span.record_exception(e)

                set_span_attribute(span, "result", result)

            except Exception as e:
                span.record_exception(e)
                span.set_status(Status(StatusCode.ERROR, str(e)))
                raise
            return result

    return traced_func



import json
from typing import Dict, List, Optional
import httpx
from .utils import parse_headers
from pydantic import BaseModel, PrivateAttr


__all__ = [
    "Thread",
    "ThreadDeleted",
    "ToolResources",
    "ToolResourcesCodeInterpreter",
    "ToolResourcesFileSearch",
]


class ToolResourcesCodeInterpreter(BaseModel, extra="allow"):
    file_ids: Optional[List[str]] = None


class ToolResourcesFileSearch(BaseModel, extra="allow"):
    vector_store_ids: Optional[List[str]] = None


class ToolResources(BaseModel, extra="allow"):
    code_interpreter: Optional[ToolResourcesCodeInterpreter] = None
    file_search: Optional[ToolResourcesFileSearch] = None


class Thread(BaseModel, extra="allow"):
    id: Optional[str] = None
    created_at: Optional[int] = None
    metadata: Optional[object] = None
    object: Optional[str] = None
    tool_resources: Optional[ToolResources] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class ThreadDeleted(BaseModel, extra="allow"):
    id: Optional[str] = None
    object: Optional[str] = None
    deleted: Optional[bool] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)
